#ifndef __MYORCA_UART_H
#define __MYORCA_UART_H

// Memory I/O macro
#define mmio32(x)   (*(volatile unsigned long *)(x))
#define mmio8(x)    (*(volatile unsigned char *)(x))

// UART Interface
#define UART_BASE    (0xE0000000)
// WO : send value via uart
#define UART_0       (UART_BASE)
// RO : read value from uart (non blocking)
#define UART_1       (UART_BASE + 0x0004)
// RO : get send fifo element count
#define UART_2       (UART_BASE + 0x0008)
// RO : get receive fifo element count
#define UART_3       (UART_BASE + 0x000C)


__attribute__((always_inline))
static inline void uart_printc(char c) {
   mmio8(UART_0) = c; // write
}

void uart_printi(int i);

// null terminated
void uart_printf(char *str);

__attribute__((always_inline))
static inline char uart_read_char() {
  return (char) mmio8(UART_1); // read
}

__attribute__((always_inline))
static inline unsigned int uart_get_send_fifo_count() {
  return (unsigned int) mmio32(UART_2); // read
}

__attribute__((always_inline))
static inline unsigned int uart_get_read_fifo_count() {
  return (unsigned int) mmio32(UART_3); // read
}

void wait(unsigned int n);



#endif //#ifndef __MYORCA_UART_H
